# A thin command-line wrapper for the Zemanta Suggest service 
# Written by Tomaz Solc, August 2008

use strict;
use open ':locale';

use Net::Zemanta::Suggest;

use Term::ReadLine;
use Getopt::Long;
use HTML::TreeBuilder;
use HTML::Element;

# Settings configurable from the command line, with default values

my $apiKey;
my $interactive = 0;
my $command;
my $inputFile;
my $outputFile;
my $help = 0;

my $version = "0.5";
my $userAgent = "Zemanta Nanoblogger/$version";

my $term = new Term::ReadLine 'Zemanta Suggest';
$term->ornaments(0);

# Smart links are only created for text inside these HTML tags.

my @smartlinkElements = ( 'p', 'ul', 'ol', 'dl' );

# User interface functions

sub query {
	my ($prompt, $default) = @_;

	print "$prompt [$default]\n";
	my $input = $term->readline(': ');

	if(length($input)<1) {
		return $default;
	} else {
		return $input;
	}
}

sub queryList {
	my $prompt = shift;
	my $list = shift;

	my @t = 1..($#$list+1);
	my $default_list = shift;
	$default_list = \@t unless( defined($default_list) );

	my @result;

	return \@result if( $#$list < 0 );

	my $strDefault = join(' ', @$default_list);

	my $r = &query($prompt, $strDefault);

	for my $c (split(/[ ,]+/, $r)) {
		if($c =~ /[0-9]+/ and defined($$list[$c-1])) {
			push(@result, $$list[$c-1]);
		}
	}

	return \@result;
}

sub querySingleList {
	my ($prompt, $list) = @_;
	my @result;

	return \@result if( $#$list < 0 );

	my $r = &query($prompt, "1");

	if($r =~ /[0-9]+/ and defined($$list[$r-1])) {
		push(@result, $$list[$r-1]);
	}

	return \@result;
}

# Shortens text to 100 characters

sub shorten {
	my ($text, $maxlen) = @_;

	$maxlen = 100 if( !defined($maxlen) );

	if( length( $text ) > $maxlen ) {
		$text = substr( $text, 0, $maxlen ) . "...";
	}

	return $text;
}

# Applies a link into a HTML::Tree

sub applyLinkRecursion {
	my ($tree, $link) = @_;

	# Never nest <a> tags
	return 0 if( $tree->tag() eq 'a' );

	my $regex;
	if( $$link{'anchor'} =~ /^\w.*\w$/ ) {
		# Anchor starts and ends with a normal word, 
		# so match only on word boundaries.
		$regex = qr/^(.*)\b(\Q$$link{'anchor'}\E)\b(.*)$/;
	} else {
		# Anchor starts or ends with punctuation. Match anywhere.
		$regex = qr/^(.*)(\Q$$link{'anchor'}\E)(.*)$/;
	}

	my $count = 0;
	for my $el ( $tree->content_list() ) {
		if( ref $el eq '' ) {
			if( $el =~ /$regex/ ) {
				my $link = [ 'a', { href  => $$link{'url'},
						    class => "zem_slink" },
						  $2 ];

				$tree->splice_content($count, 1, $1, $link, $3);
				return 1;
			}
		} else {
			return 1 if( &applyLinkRecursion( $el, $link ) );
		}
		$count ++;
	}

	return 0;
}

sub applyLink {
	my ($tree, $link) = @_;

	for my $el ( $tree->find(@smartlinkElements) ) {
		return if( &applyLinkRecursion($el, $link) );
	}
}

# Takes HTML::Tree and returns the HTML inside the <body> tag.

sub extractBody {
	my ($tree) = @_;

	my $retval = '';
	for my $el ( $tree->find('body')->content_list() ) {
		if( ref $el eq '' ) {
			$retval .= $el;
		} else {
			$retval .= $el->as_XML();
		}
	}

	return $retval;
}

# Request handlers

sub keywords {
	my ($text, $r) = @_;

	my $keywords = $r->{'keywords'};

	my @s = ();
	for my $keyword (@$keywords) {
		push(@s, $$keyword{'name'});
	}

	my $suggestion = join(', ', @s);
	my $userChoice;

	if($interactive) {
		$userChoice = &query("choose keywords for your post", $suggestion);
	} else {
		$userChoice = $suggestion;
	}

	return $userChoice . "\n";
}

sub image {
	my ($text, $r) = @_;

	my $images = $r->{'images'};
	my $userChoice;

	if($interactive) {
		my $n = 1;
		for my $image (@$images) {
			last if( $n > 8 );

			$$image{'shortdesc'} = &shorten( $$image{'description'},
							 120 );

			my $license = $$image{'license'};
			$license =~ s/<[^>]*>//g;
			printf("%2d  %s\n", $n, &shorten( $$image{'description'},
							  73 ) );

			print("      $license\n");
			# print("      $$image{'url_l'}\n");
			print("      $$image{'source_url'}\n");
			
			$n++;
		}

		$userChoice = querySingleList("choose an image for your post", $images);
	} else {
		$userChoice = $images;
	}

	for my $image (@$userChoice) {
		my $urlLarge = $image->{'url_l'};
		my $urlThumb = $image->{'url_m'};
		my $description = $image->{'shortdesc'};

		my $attr = HTML::TreeBuilder->new_from_content( 
						$image->{'attribution'} );

		my $tree = HTML::Element->new_from_lol(
			[ 'div',
			  { class => 'zemanta-img' },
			  [ 'a',
			    { href => $urlLarge },
			    [ 'img',
			      { src => $urlThumb,
			        alt => $description }
			    ]
			  ],
			  [ 'p',
			    { class => "zemanta-img-attrib" },
			    $attr->find('body')->content_list()
		  	  ]
			]
			);

		my $retval = $tree->as_XML();

		$tree->delete();
		$attr->delete();

		return $retval;
	}
}

sub smartLinks {
	my ($text, $r) = @_;

	my $links = $r->{'markup'}->{'links'};

	my $userChoice;

	my $flat_links = [ ];
	my $default_list = [ ];

	my $n = 1;
	for my $link (@$links) {

		push(@$default_list, $n);

		my $targets = $link->{'target'};
		for my $target (@$targets) {
			push(@$flat_links,
					{ 'title', $target->{'title'}, 
					  'anchor', $link->{'anchor'},
					  'url', $target->{'url'} } );
			$n++;
		}
	}

	if($interactive) {
		my $n = 1;
		my $prevtitle = "";
		for my $link (@$flat_links) {
			if($$link{'title'} ne $prevtitle) {
				printf("    %s\n", $$link{'title'});
				$prevtitle = $$link{'title'};
			} 
			printf("%2d    %s\n", $n, $$link{'url'});
			$n++;
		}

		$userChoice = queryList("choose in-text links for your post", 
					$flat_links, $default_list);
	} else {
		$userChoice = $flat_links;
	}
	
	# Prepend <p> tag if we got a plain-text, since we only apply
	# links to text inside paragraphs.
	$text = "<p>" . $text unless( $text =~ /<p>/ );

	my $tree = HTML::TreeBuilder->new_from_content($text);

	for my $link (@$userChoice) {
		&applyLink($tree, $link);
	}

	my $retval = &extractBody($tree);

	$tree->delete();

	return $retval;
}

sub relatedLinks {
	my ($text, $r) = @_;

	my $articles = $r->{'articles'};
	my $userChoice;

	if($interactive) {
		my $n = 1;
		for my $article (@$articles) {
			my $url = $$article{'url'};
			$url =~ s/http:\/\/([^\/]*).*/$1/;
			print("$n $$article{'title'}\n");
			print("    $url\n");
			$n++;
		}

		$userChoice = queryList("choose related links for your post", $articles);
	} else {
		$userChoice = $articles;
	}

	my $res = "<ul>\n";
	for my $article (@$userChoice) {
		$res .= "<li><a href=\"" . $$article{'url'} . "\">";
		$res .= $$article{'title'};
		$res .= "</a></li>\n";
	}
	$res .= "</ul>\n";

	return $res;
}

sub syntax {
	print <<END
USAGE: zemanta.pl [ OPTION ] ... QUERY [ FILE ]

Execute a query of your choice on the Zemanta Suggest system. Article text,
assumed to be encoded in UTF-8, is read from FILE (or from standard input
if FILE is ommited).

Available queries: image, keywords, relatedlinks, smartlinks

Available options:
  -i, --interactive                 be interactive
      --key=API-KEY                 API key to use
  -o, --output=FILE                 write output to FILE instead of
                                    standard output

  -h, --help                        print this help message
  -V, --version                     print version and licence information
END
	;
	exit(1);
}

sub legal {
	print <<END
Zemanta Suggest command line wrapper, version $version
END
	;
	exit(0)
}

# Parse command line options and print out help message we got a combination
# of options we didn't understand.

&syntax() if not GetOptions(	'key=s' 		=> \$apiKey,
				'i|interactive!'	=> \$interactive,
				'o|output=s'		=> \$outputFile,
				'h|help'		=> \$help );

&syntax if($#ARGV != 0 and $#ARGV != 1);
&syntax() if($help);

$command = $ARGV[0];

my $text = "";

# Load input into $text. Either from a file or from standard input

if($#ARGV == 1) {
	$inputFile = $ARGV[1];

	open(INPUT, "<$inputFile") or die("Can't open $inputFile: $!");
	$text .= $_ while( <INPUT> );
	close(INPUT);
} else {
	# Can't be interactive and read data from standard input.
	$interactive = 0;
	$text .= $_ while( <STDIN> );
}

# Create Zemanta Suggest object
my $zemanta = Net::Zemanta::Suggest->new(
			APIKEY     => $apiKey,
			USER_AGENT => $userAgent );

my $suggestions = $zemanta->suggest($text);
unless( defined($suggestions) ) {
	print STDERR $zemanta->error(), "\n";
	exit(1);
}

# Process the request

my $r;

if($command eq "image") {
	$r = &image($text, $suggestions);
} elsif($command eq "keywords") {
	$r = &keywords($text, $suggestions);
} elsif($command eq "relatedlinks") {
	$r = &relatedLinks($text, $suggestions);
} elsif($command eq "smartlinks") {
	$r = &smartLinks($text, $suggestions);
} else {
	&syntax();
}

# Print out the result

if(defined $outputFile) {
	open(OUTPUT, ">$outputFile") or die("Can't open $outputFile: $!");
	print OUTPUT $r;
	close(OUTPUT)
} else {
	print $r;
}

exit(0);
